!{\src2tex{textfont=tt}}
!!****f* ABINIT/aprxdr
!! NAME
!! aprxdr
!!
!! FUNCTION
!! Compute the approximative derivatives of the energy at different
!! points along the line search, thanks to a finite-difference formula.
!! This formula is the projection along the line search of the
!! Eq.(11) in PRB54, 4383 (1996).
!!
!! COPYRIGHT
!! Copyright (C) 1998-2003 ABINIT group (XG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!! cplex: if 1, real space functions on FFT grid are REAL, if 2, COMPLEX
!! choice= if==3, compute dedv_new, dedv_old, and dedv_mix,
!! if/=3, compute only dedv_new and dedv_old.
!! i_vresid and i_rhor, see the next lines.
!! f_fftgr(nfft,nspden,n_fftgr)=different functions defined on the fft grid :
!! The last residual potential is in f_fftgr(:,:,i_vresid(1)).
!! The old  residual potential is in f_fftgr(:,:,i_vresid(2)).
!! The previous old residual potential is in f_fftgr(:,:,i_vresid(3)).
!! (needed only when choice==3)
!! The old  density is in f_fftgr(:,:,i_rhor2).
!! f_atm(3,natom,n_fftgr)=different functions defined for each atom :
!! The last HF force is in f_atm(:,:,i_vresid(1)).
!! The old  HF force is in f_fftgr(:,:,i_vresid(2)).
!! The previous old HF force is in f_fftgr(:,:,i_vresid(3)).
!! (needed only when choice==3)
!! The old atomic positions are in f_atm(:,:,i_rhor2)
!! moved_atm_inside: if==1, the atoms are allowed to move.
!! natom=number of atoms in unit cell
!! nfft=number of fft grid points
!! nspden=number of spin-density components
!! rhor(nfft,nspden)=actual density
!! xred(3,natom)=reduced atomic coordinates
!!
!! OUTPUT
!! dedv_mix=approximate derivative from previous old residual
!! dedv_new=approximate derivative from new residual
!! dedv_old=approximate derivative from old residual (output only when choice==3)
!!
!! NOTES
!! Should be OpenMP parallelized
!!
!! PARENTS
!!      scfcge
!!
!! CHILDREN
!!
!! SOURCE

 subroutine aprxdr(cplex,choice,dedv_mix,dedv_new,dedv_old,&
&  f_atm,f_fftgr,i_rhor2,i_vresid,moved_atm_inside,&
&  natom,nfft,nspden,n_fftgr,rhor,ucvol,xred)

 use defs_basis
 
 implicit none
 
!Arguments ------------------------------------
 integer :: cplex,choice,iout,i_rhor2,moved_atm_inside,&
& natom,nfft,nspden,n_fftgr
 integer :: i_vresid(3)
 real(dp) :: dedv_mix,dedv_new,dedv_old,ucvol
 real(dp) :: f_atm(3,natom,n_fftgr),&
&  f_fftgr(cplex*nfft,nspden,n_fftgr),rhor(cplex*nfft,nspden),xred(3,natom)
 
!Local variables-------------------------------
 integer :: iatom,idir,ifft
 real(dp) :: diff_rhor_down,diff_rhor_up
 real(dp) :: diff_rhor(4)
 
! *************************************************************************
 
 dedv_old=0.0d0
 dedv_new=0.0d0
 if(choice==3)dedv_mix=0.0d0

!Compute approximative derivative of the energy
!with respect to change of potential

 if(nspden==1)then

  do ifft=1,cplex*nfft 
   dedv_old=dedv_old+&
&     f_fftgr(ifft,1,i_vresid(2))*(rhor(ifft,1)-f_fftgr(ifft,1,i_rhor2))
   dedv_new=dedv_new+&
&     f_fftgr(ifft,1,i_vresid(1))*(rhor(ifft,1)-f_fftgr(ifft,1,i_rhor2))
   if(choice==3)then
    dedv_mix=dedv_mix+&
&     f_fftgr(ifft,1,i_vresid(3))*(rhor(ifft,1)-f_fftgr(ifft,1,i_rhor2))
   endif
  enddo ! ifft

 else if(nspden==2) then

  do ifft=1,cplex*nfft 
!  This is made strange because of the different convention for storage
!  in vresid and rho in the spin-polarized case
   diff_rhor_up=rhor(ifft,2)-f_fftgr(ifft,2,i_rhor2)
   diff_rhor_down=rhor(ifft,1)-f_fftgr(ifft,1,i_rhor2)-diff_rhor_up
   dedv_old=dedv_old +f_fftgr(ifft,1,i_vresid(2))*diff_rhor_up        &
&                    +f_fftgr(ifft,2,i_vresid(2))*diff_rhor_down
   dedv_new=dedv_new +f_fftgr(ifft,1,i_vresid(1))*diff_rhor_up        &
&                    +f_fftgr(ifft,2,i_vresid(1))*diff_rhor_down
   if(choice==3)then
    dedv_mix=dedv_mix +f_fftgr(ifft,1,i_vresid(3))*diff_rhor_up   &
&                     +f_fftgr(ifft,2,i_vresid(3))*diff_rhor_down
   endif
  enddo

 else if(nspden==4) then

! $de= V^{\alpha,\beta} d \rho{\alpha,\beta}=
!       d m_x Re(V^{12})-d m_y Im{V^{12}}+d m_z(V^{11}-V^{22})/2
!      +d rho*(V^{11}+V^{22})/2$
! Check-it !
  do ifft=1,cplex*nfft 
   diff_rhor(:)=rhor(ifft,:)-f_fftgr(ifft,:,i_rhor2)
   dedv_old=dedv_old &
&   +(f_fftgr(ifft,1,i_vresid(2))+f_fftgr(ifft,2,i_vresid(2)))*diff_rhor(1)*half &
&   +(f_fftgr(ifft,1,i_vresid(2))-f_fftgr(ifft,2,i_vresid(2)))*diff_rhor(4)*half &
&   + f_fftgr(ifft,3,i_vresid(2))*diff_rhor(2) &
&   - f_fftgr(ifft,4,i_vresid(2))*diff_rhor(3) !-mx suivant xg
   dedv_new=dedv_new &
&   +(f_fftgr(ifft,1,i_vresid(1))+f_fftgr(ifft,2,i_vresid(1)))*diff_rhor(1)*half &
&   +(f_fftgr(ifft,1,i_vresid(1))-f_fftgr(ifft,2,i_vresid(1)))*diff_rhor(4)*half &
&   + f_fftgr(ifft,3,i_vresid(1))*diff_rhor(2) &
&   - f_fftgr(ifft,4,i_vresid(1))*diff_rhor(3) 
   if(choice==3)then
    dedv_mix=dedv_mix &
&    +(f_fftgr(ifft,1,i_vresid(3))+f_fftgr(ifft,2,i_vresid(3)))*diff_rhor(1)*half &
&    +(f_fftgr(ifft,1,i_vresid(3))-f_fftgr(ifft,2,i_vresid(3)))*diff_rhor(4)*half &
&    + f_fftgr(ifft,3,i_vresid(3))*diff_rhor(2) &
&    - f_fftgr(ifft,4,i_vresid(3))*diff_rhor(3) 
   endif
  enddo
 endif

!cplex is not included in the normalisation, of course
 dedv_old=dedv_old * ucvol/dble(nfft)
 dedv_new=dedv_new * ucvol/dble(nfft)
 if(choice==3)dedv_mix=dedv_mix * ucvol/dble(nfft)

! -------------------------------------------------------

!Now, take care of eventual atomic displacements

 if(moved_atm_inside==1)then
  do idir=1,3
   do iatom=1,natom
    dedv_new=dedv_new+&
&    f_atm(idir,iatom,i_vresid(1))*(xred(idir,iatom)-f_atm(idir,iatom,i_rhor2)) 
    dedv_old=dedv_old+&
&    f_atm(idir,iatom,i_vresid(2))*(xred(idir,iatom)-f_atm(idir,iatom,i_rhor2))
    if(choice==3) dedv_mix=dedv_mix+&
&    f_atm(idir,iatom,i_vresid(3))*(xred(idir,iatom)-f_atm(idir,iatom,i_rhor2))
   enddo
  enddo 
 endif

 end subroutine
!!***
